﻿' VB.NET to Access Header User Data
'   lines starting with an apostrophe are comments
Public Class HeaderUD

    ' create Visual CADD object
    Shared WithEvents vcadd As New VCadd32.Automation

    Public Shared Function Run(ByVal str As String) As Integer

        '   Set the required unique user data name in the application
        '   initialization routine:
        '       In order to access user data for each entity and the 
        '       drawing header, a unique user data name must be 
        '       specified to reference the information.  The user 
        '       data name is case insensitive and can be up to 15 
        '       characters long.  The following code sets a user 
        '       data name of "MyData":

        vcadd.SetUserDataName("MyData")

        '   Access the user data in the drawing header:
        '       Access to user data in the drawing header is 
        '       identical to the user data in the current entity 
        '       except that you call VCSetHeaderUserData instead of 
        '       VCSetEntityUserData and there is no current entity.

        vcadd.SetHeaderUserData()

        '   Add or set the desired user data values to the 
        '   drawing header:
        '       The Visual CADD API allows two methods for 
        '       attaching the user data information.  The easiest
        '       method is to simply use the add routines which 
        '       adds the data to the next available slot in the 
        '       user data segment. The API also allows user data 
        '       to be set and placed at a specific index within 
        '       the user data.  The benefits of the second method 
        '       become apparent when working with multiple data 
        '       values being attached to the header.  As more 
        '       data is attached, a hierarchical system can be 
        '       set up to access the information desired.  The 
        '       following code adds a user data short (0), 
        '       double (25.5), string ("My Application Data"), and 
        '       chunk to the drawing header:

        vcadd.AddCurrentEntityUserDataShort(0)
        vcadd.AddCurrentEntityUserDataDouble(25.5)
        vcadd.AddCurrentEntityUserDataString("My Application Data")

        Dim bFlags As Byte() = {&H23, &HA3, &H6F, &H8E, &H66, &HFF, &HFF, &HB2}
        vcadd.AddCurrentEntityUserDataChunk(bFlags)

        '   Optional - if the drawing header already has user 
        '   data and your application knows the order and kind of
        '   the data, then you can set or replace the existing 
        '   data.  The following code sets the previously added 
        '   user data short (31) and double (-52.2) at index 0 
        '   and 1, respectively:

        vcadd.SetCurrentEntityUserDataShort(0, 31)
        vcadd.SetCurrentEntityUserDataDouble(1, -52.2)

        '   Retrieve the user data count for the unique user 
        '   data ID:
        '       This count returns the number of valid user data 
        '       segments attached to the header.  This 
        '       information can then used to loop through the 
        '       retrieval process.  The following code retrieves 
        '       the amount of data attached at the current 
        '       location:

        Dim iUDCount As Short
        iUDCount = vcadd.GetCurrentEntityUserDataCount()

        '   For each index, retrieve the type of user data 
        '   information attached.  This step can be avoided if 
        '   the application remembers and accesses the 
        '   information all in the same order each time.  The 
        '   following code retrieves the user data kind at the 
        '   specified index.  Typically this would be done in a 
        '   loop statement to check all the data on the entity:

        For iIndex = 0 To iUDCount - 1
            '   If the data type is not known then typically this 
            '   would involve a statement to retrieve the user 
            '   data kind then retrieve the value of the 
            '   appropriate type.  If the application remembers 
            '   and accesses the user data the same way each 
            '   time, you would just make direct calls to the 
            '   corresponding functions for the respective index.

            Dim iUDKind As Short
            iUDKind = vcadd.GetCurrentEntityUserDataKind(iIndex)

            Select Case iUDKind
                Case USER_BYTE
                    Dim bByte As Byte
                    bByte = vcadd.GetCurrentEntityUserDataByte(iIndex)
                Case USER_SHORT
                    Dim nShort As Short
                    nShort = vcadd.GetCurrentEntityUserDataShort(iIndex)
                Case USER_LONG
                    Dim iLong As Long
                    iLong = vcadd.GetCurrentEntityUserDataLong(iIndex)
                Case USER_DOUBLE
                    Dim dDouble As Double
                    dDouble = vcadd.GetCurrentEntityUserDataDouble(iIndex)
                Case USER_FLOAT
                    Dim fFloat As Single
                    fFloat = vcadd.GetCurrentEntityUserDataFloat(iIndex)
                Case USER_CHUNK
                    Dim iSize As Short
                    iSize = vcadd.GetCurEntUserDataChunkSize(iIndex)
                    Dim pBytes(iSize - 1)
                    pBytes = vcadd.GetCurrentEntityUserDataChunk(iIndex)
                    For i = 0 To iSize - 1
                        Dim b As Byte = pBytes(i) ' access the array
                    Next
                Case USER_STRING
                    Dim sz As String
                    sz = vcadd.GetCurrentEntityUserDataString(iIndex)
            End Select
        Next

        ' return
        Return (True)

    End Function

End Class
