﻿' VB.NET to Retrieve User Data
'   lines starting with an apostrophe are comments
Public Class RetrieveUD

    ' create Visual CADD object
    Shared WithEvents vcadd As New VCadd32.Automation

    Public Shared Function Run(ByVal str As String) As Integer

        '   Set the required unique user data name in the application
        '   initialization routine:
        '       In order to access user data for each entity and the 
        '       drawing header, a unique user data name must be 
        '       specified to reference the information.  The user 
        '       data name is case insensitive and can be up to 15 
        '       characters long.  The following code sets a user 
        '       data name of "MyData":

        vcadd.SetUserDataName("MyData")

        '   Retrieve the information to the current entity:
        '       To access user data in the current entity, call 
        '       VCSetEntityUserData.

        vcadd.SetEntityUserData()

        '     The current entity can be set in a number of ways 
        '     using any of the Visual CADD API parsing routines or 
        '     VCSetCurrentEntity.  For information see the topic 
        '     Parsing the Database.  For illustration, we will use 
        '     the first entity in the database with entity handle or 
        '     index 0.  The user data will be retrieved from this 
        '     current entity.

        vcadd.SetCurrentEntity(0)

        '   verify we have a current entity before proceeding
        If (vcadd.GetErrorCode() = 0) Then

            '   Retrieve the user data count for the unique user 
            '   data ID:
            '       This count returns the number of valid user data 
            '       segments attached to the entity.  This 
            '       information can then used to loop through the 
            '       retrieval process.  The following code retrieves 
            '       the amount of data attached at the current 
            '       location:

            Dim iUDCount As Short
            iUDCount = vcadd.GetCurrentEntityUserDataCount()

            '   For each index, retrieve the type of user data 
            '   information attached.  This step can be avoided if 
            '   the application remembers and accesses the 
            '   information all in the same order each time.  The 
            '   following code retrieves the user data kind at the 
            '   specified index.  Typically this would be done in a 
            '   loop statement to check all the data on the entity:

            For iIndex = 0 To iUDCount - 1
                '   If the data type is not known then typically this 
                '   would involve a statement to retrieve the user 
                '   data kind then retrieve the value of the 
                '   appropriate type.  If the application remembers 
                '   and accesses the user data the same way each 
                '   time, you would just make direct calls to the 
                '   corresponding functions for the respective index.

                Dim iUDKind As Short
                iUDKind = vcadd.GetCurrentEntityUserDataKind(iIndex)

                Select Case iUDKind
                    Case USER_BYTE
                        Dim bByte As Byte
                        bByte = vcadd.GetCurrentEntityUserDataByte(iIndex)
                    Case USER_SHORT
                        Dim nShort As Short
                        nShort = vcadd.GetCurrentEntityUserDataShort(iIndex)
                    Case USER_LONG
                        Dim iLong As Long
                        iLong = vcadd.GetCurrentEntityUserDataLong(iIndex)
                    Case USER_DOUBLE
                        Dim dDouble As Double
                        dDouble = vcadd.GetCurrentEntityUserDataDouble(iIndex)
                    Case USER_FLOAT
                        Dim fFloat As Single
                        fFloat = vcadd.GetCurrentEntityUserDataFloat(iIndex)
                    Case USER_CHUNK
                        Dim iSize As Short
                        iSize = vcadd.GetCurEntUserDataChunkSize(iIndex)
                        Dim pBytes(iSize - 1)
                        pBytes = vcadd.GetCurrentEntityUserDataChunk(iIndex)
                        For i = 0 To iSize - 1
                            Dim b As Byte = pBytes(i) ' access the array
                        Next
                    Case USER_STRING
                        Dim sz As String
                        sz = vcadd.GetCurrentEntityUserDataString(iIndex)
                End Select
            Next
        End If

        ' return
        Return (True)

    End Function

End Class
